package abstractions;

import interfaces.ElevatorForUsersSpecification;
import interfaces.ElevatorUserSpecification;

import java.util.Random;

import tools.HasInvariant;
import enumerations.UserState;

/**
 * Implements the common behaviour of all elevator users as specified by {@link ElevatorUserSpecification}<br>
 * The class is abstract because it does not implement the variant behaviour of different types of user
 * associated with the method {@link ElevatorUserSpecification#makeDecision()}. <br><br>
 * 
 * @author jpaulgibson
 */
public abstract class ElevatorUser implements ElevatorUserSpecification, HasInvariant{

	
	/**
	 * The elevator user can be in 1 of the states as defined by {@link enumerations.UserState}
	 */
	protected UserState state;
	
	/**
	 * The elevator that the user has decided to use
	 */
	protected ElevatorForUsersSpecification elevator;
	
	/**
	 * The next unique integer identifier to be allocated to a user; starting from 0
	 */
	private static int next_ID =0;
	
	/**
	 * The unique identifier allocated to the user
	 */
	public final int unique_ID;
	
	/**
	 * the floor from which the user wishes to be collected by the elevator
	 */
	protected final int startFloor;
	
	/**
	 * the floor to which the user wishes to be taken by the elevator
	 */
	protected final int endFloor;
	
	/**
	 * 
	 * @param startFloor is the floor from which the user wishes to be collected by the elevator
	 * @param endFloor is the floor to which the user wishes to be taken by the elevator
	 * @param elevator is the elevator that the user has decided to use
	 * @throws IllegalArgumentException if the elevator assigned to the user is null
	 * @throws IllegalArgumentException if the floors specified are negative, or if they are not different
	 * @throws IllegalStateException if we have reached the maximum permitted number of users 
	 *         (as defined by {@link Integer#MAX_VALUE})
	 */
	public ElevatorUser (int startFloor, int endFloor, ElevatorForUsersSpecification elevator) throws IllegalArgumentException, IllegalStateException{
		
		if (elevator == null)
			throw new IllegalArgumentException ("The elevator assigned to a user must not be null");
		
		if (startFloor == endFloor)
			throw new IllegalArgumentException ("Start and end floors must be different");
		
		if (startFloor <0 )
			throw new IllegalArgumentException ("Start floor must not be negative");
		
		if (endFloor <0 )
			throw new IllegalArgumentException ("End floor must not be negative");
		
		if (startFloor > elevator.topFloor() )
			throw new IllegalArgumentException ("Start floor must not be above the top floor");
		
		if (endFloor > elevator.topFloor())
			throw new IllegalArgumentException ("End floor must not be above the top floor");
		
		if (next_ID <0) 
			throw new IllegalStateException ("Reached limit of the number of Elevator Users ");
		
		state = UserState.ARRIVED_AT_FLOOR;
		
		unique_ID = next_ID;
		next_ID++;
		
		this.elevator = elevator;
		this.startFloor = startFloor;
		this.endFloor = endFloor;	
	}
	
	/**
	 * Make a random elevator user where source and destination floors are chosen randomly
	 * @param RNG the random number generator to use when assigning the floor values
	 * @param elevator the elevator that the user will be using
	 * @throws IllegalArgumentException if the elevator assigned to the user is null
	 * @throws IllegalStateException if we have reached the maximum permitted number of users 
	 *         (as defined by {@link Integer#MAX_VALUE})
	 */
	public ElevatorUser (Random RNG, ElevatorForUsersSpecification elevator) throws IllegalArgumentException, IllegalStateException{
		
		if (elevator == null)
			throw new IllegalArgumentException ("The elevator assigned to a user must not be null");
		
		if (next_ID <0) 
			throw new IllegalStateException ("Reached limit of the number of Elevator Users ");

		int from;
		int to;
		
		state = UserState.ARRIVED_AT_FLOOR;
		
	    from = RNG.nextInt(elevator.topFloor()+1);
		do to = RNG.nextInt(elevator.topFloor()+1);
		   while (to==from);
		
		unique_ID = next_ID;
		next_ID++;
		
		this.elevator = elevator;
		this.startFloor = from;
		this.endFloor = to;
	}
	
	@Override
	public void pressedButtonAtFloor() throws IllegalStateException{
		
		if (state != UserState.ARRIVED_AT_FLOOR) throw new IllegalStateException ("User not arrived at floor");
		else  state =  UserState.WAITING_AT_FLOOR;
	}
		
		
	@Override
	public void pickedUpAtFloor() throws IllegalStateException{
		
		if (state != UserState.WAITING_AT_FLOOR) throw new IllegalStateException ("User not waiting at floor");
		else  state =  UserState.IN_ELEVATOR;
		
	}
	
	@Override
	public void deliveredToFloor() throws IllegalStateException{
		
		if (state != UserState.IN_ELEVATOR) throw new IllegalStateException ("User not inside the elevator");
		else  state =  UserState.SERVICED;
		
	}
	
	@Override
	public boolean invariant (){
		 return (startFloor!=endFloor && startFloor>=0 && endFloor >= 0 && startFloor<= elevator.topFloor() && endFloor<= elevator.topFloor());
	}
	
	@Override
	public int hashCode (){
		
		return unique_ID;
	}
	
	public boolean equals (Object obj){
		
		if (obj == null) return false;
		if (this == obj) return true;
		if (! (obj instanceof ElevatorUser)) return false;
		ElevatorUser u = (ElevatorUser) obj;
		return this.unique_ID == u.unique_ID;
		
	}
	

	@Override
	public UserState get_state() {
		
		return state;
	}

	@Override
	public int get_to() {
		return endFloor;
	}

	@Override
	public int get_from() {
		return startFloor;
	}
		
		
}