package abstractions;

import models.elevatorButtons.FloorButtonsInElevator;

/**
 * Adds security to an elevator by allowing it to be installed with some floors that are lockable.
 * When a floor is locked then the floor request button inside the lift is disabled. This
 * decision means that users already at locked floors do not need to unlock the floor
 * in order to request the lift to come to collect them.<br><br>
 * 
 * For safety reasons, the bottom floor (0) must never be lockable. <br><br>
 * 
 * To access a locked floor from within an elevator, the floor chosen must be unlocked before the
 * floor buttons become enabled. When a floor is requested that is locked, the user receives a warning
 * telling them that the request cannot be fulfilled. <br><br>
 * 
 * 
 * The class is abstract because there are alternative possible behaviours for when a floor is locked 
 * (see {@link #lock(int )}), eg:
 * <ul>
 * <li> {@link models.elevatorButtons.LockableFloorButtonsInElevator_Option1} 
 * <li> {@link models.elevatorButtons.LockableFloorButtonsInElevator_Option2} 
 * <li> ...
 * </ul>
 * So we chose to leave this open as an option in the concrete implementation.
 * 
 * @author jpaulgibson
 */
public abstract class LockableFloorButtonsInElevator extends FloorButtonsInElevator{
	
	/**
	 * Which floors are lockable
	 */
	protected final boolean [] lockableFloor;
	
	
	/**
	 * Which floors are locked
	 */
	protected  boolean [] lockedFloor;
	
    /**
     * Additional safety requirements are:
     * <ul>
     * <li> The {@link #lockableFloor} array length is the same as the {@link FloorButtonsInElevator#NUMBER_OF_FLOORS} </li>
     * <li> The maximum number of locked floors is the same as the number of floors in the elevator. </li>
     * <li> Only lockable floors can be locked </li>
     * </ul>
     */
	public boolean invariant(){
		
		if (!super.invariant()) return false;
		
		if (lockableFloor.length != NUMBER_OF_FLOORS) return false;
		
		if (lockedFloor.length != NUMBER_OF_FLOORS) return false;
		
       // Only lockable floors can be locked
		for (int floor =0; floor< NUMBER_OF_FLOORS; floor++)
			if (lockedFloor[floor] && !lockableFloor[floor] ) return false;

		return true;
	}
	
	public LockableFloorButtonsInElevator (int numberOfFloors, boolean [] lockableFloor ){
		
		super(numberOfFloors);
		this.lockableFloor = lockableFloor;
		lockedFloor = new boolean [numberOfFloors];
		for (int floor =0; floor<numberOfFloors; floor++)
			 if (lockableFloor[floor]) lockedFloor[floor] = true; else lockedFloor[floor] = false;
	}
	
	/**
	 * Place a lockable floor in a locked state <br>
	 * There may be different security policies concerning what to do when a floor that was unlocked is locked again<br>
	 * Thus, this is an abstract method which must be implemented by a concrete policy  in any concrete subclass
	 * @param floor is the floor to lock
	 * @throws IllegalArgumentException if the floor is not valid
	 * @throws IllegalArgumentException if the floor is not lockable
	 */
    public abstract void lock (int floor) throws IllegalArgumentException;
    
    /**
     * Place a lockable floor in an unlocked state
     * @param floor is the floor to unlock
	 * @throws IllegalArgumentException if the floor is not valid
	 * @throws IllegalArgumentException if the floor is not (un)lockable
     */
    public  void unlock (int floor) throws IllegalArgumentException{
    	
    	if (floor<0 || floor > NUMBER_OF_FLOORS)
    		throw new IllegalArgumentException("The floor is not valid");
    	
    	if (!lockableFloor[floor])
    		throw new IllegalArgumentException("The floor is not (un)lockable");
    	
    	else lockedFloor[floor] = false;
    }
    
    /**
     * Specialises the behaviour of {@link FloorButtonsInElevator#pressFloorInElevator(int)} by
     * not enabling requests inside the elevator for floors that are locked.
     * Same behaviour as {@link #tryToPressFloorInElevator (int )}, except that instead of returning a
     * boolean value  to inform the client if the request was carried out, we display
     * a message if the floor request was not enabled. 
     * @throws IllegalArgumentException if the floor is not valid
     */
    @Override
     public void pressFloorInElevator (int floor) throws IllegalArgumentException{
    	
    	if (floor<0 || floor > NUMBER_OF_FLOORS)
    		throw new IllegalArgumentException("The floor is not valid");
    	 
    	 if (!lockedFloor[floor]) super.pressFloorInElevator(floor);
    	 else System.out.println("Floor "+floor+" is locked and your request cannot be authorized.");
     }
     
      /**
       * Same behaviour as {@link #pressFloorInElevator (int )}, except that instead of displaying
       * a message if the floor request was not enabled, a boolean value is returned to 
       * inform the client if the request was carried out.
       * @param floor is the target destination of the user who wishes to press the button
       * @return if the request was authorized and enabled
       * @throws IllegalArgumentException if the floor is not valid
       */
      public boolean tryToPressFloorInElevator (int floor) throws IllegalArgumentException{
    	  
      	if (floor<0 || floor > NUMBER_OF_FLOORS)
    		throw new IllegalArgumentException("The floor is not valid");
     	 
     	 if (!lockedFloor[floor]){
     		 super.pressFloorInElevator(floor);
     		 return true;
     	 }
     	 else return false;
      }
     
      /**
       * 
       * @param floor to be checked
       * @return if it is lockable
       * @throws IllegalArgumentException if the floor is not valid
       */
     public boolean is_lockable(int floor){
    	 
    	 if (floor<0 || floor > NUMBER_OF_FLOORS)
     		throw new IllegalArgumentException("The floor is not valid");
    	 
     	if (floor<0 || floor > NUMBER_OF_FLOORS)
    		throw new IllegalArgumentException("The floor is not valid");
    	 
    	 return lockableFloor[floor];
     }
     
     /**
      * 
      * @param floor to be checked 
      * @return if it is locked
      * @throws IllegalArgumentException if the floor is not valid
      */
     public boolean is_locked(int floor){
    	 
    	 if (floor<0 || floor > NUMBER_OF_FLOORS)
     		throw new IllegalArgumentException("The floor is not valid");
    	 
     	if (floor<0 || floor > NUMBER_OF_FLOORS)
    		throw new IllegalArgumentException("The floor is not valid");
    	 
    	 return lockedFloor[floor];
     }
     
    
 	public String toString(){
 		String str = super.toString();
		 str= str +"\nLockable Floors In Elevator:";
		for (int floor=0; floor< NUMBER_OF_FLOORS;floor++){
			
			if (lockableFloor[floor]){
				str = str + " "+floor+":";
				if (lockedFloor[floor]) str = str + "L"; else str = str + "U";
			}
			else str = str +" ";
		}
		return str;
		
	}
     
}
