package factoriesSPL;

import interfaces.ElevatorForUsersSpecification;
import interfaces.ElevatorUserSpecification;

import java.util.Random;

import models.elevatorUsers.IntelligentElevatorUser;
import models.elevatorUsers.IntelligentElevatorUserWithKey;
import models.elevatorUsers.PressAllButtonsElevatorUser;
import enumerations.UserType;

/**
 * This factory for making elevator users - as specified in {@link interfaces.ElevatorUserSpecification} -
 * is implemented as a singleton as it allows only one instantiation.<br>
 * 
 * The code follows Java specific design proposed by Bill Pugh 
 * from <em> Concurrent  programming in Java: design principles and patterns</em><br>
 * @author jpaulgibson
 *
 */
public class ElevatorUserFactory {
	
	/**
	 * The default user type used in the default user constructor
	 */
	public final static UserType DEFAULT_USER_TYPE = UserType.INTELLIGENT;
	
	/** 
	 * The default elevator used in the default user constructor
	 */
	public final static ElevatorForUsersSpecification DEFAULT_ELEVATOR = ElevatorFactory.makeElevator();
	
	/**
	 * Unless specified, the default user starts their journey on the bottom floor
	 */
	public final static int DEFAULT_FROM = 0;
	
	/**
	 * Unless specified, the default user wishes to travel to the top floor
	 */
	public final static int DEFAULT_TO = DEFAULT_ELEVATOR.topFloor();
	
	private ElevatorUserFactory() {}
	 
	  private static class SingletonHolder { 
	    private static final ElevatorUserFactory INSTANCE = new ElevatorUserFactory();
	  }

	  public static ElevatorUserFactory getInstance() {
	    return SingletonHolder.INSTANCE;
	  }
	
	/**
	 * Tested by {@link unit_tests.factoriesSPL.JUnit_ElevatorUserFactory#test_makeDefaultElevatorUser()}.<br>
	 * Default constructor for an elevator user 
	 * @return a user configured as follows:
	 * <ul> 
	 * <li> {@link #DEFAULT_USER_TYPE} </li>
	 * <li> {@link #DEFAULT_ELEVATOR} </li>
	 * <li> {@link #DEFAULT_FROM} </li>
	 * <li> {@link #DEFAULT_TO} </li>
	 * </ul>
	 */
	public  ElevatorUserSpecification makeElevatorUser(){
		
		return makeElevatorUser(DEFAULT_USER_TYPE, DEFAULT_FROM, DEFAULT_TO, DEFAULT_ELEVATOR);	
	}
	
	
	/**
	 * Elevator user is constructed precisely defined as by the configuration parameters provided<br>
	 * Tested by 
	 * <ul>
	 * <li> {@link unit_tests.factoriesSPL.JUnit_ElevatorUserFactory#test_makeIntelligentElevatorUser_OK()}  </li>
	 * <li> {@link unit_tests.factoriesSPL.JUnit_ElevatorUserFactory#test_makeIntelligentElevatorUser_KO1()} </li>
	 * <li> {@link unit_tests.factoriesSPL.JUnit_ElevatorUserFactory#test_makeIntelligentElevatorUser_KO2()} </li>
	 * <li> {@link unit_tests.factoriesSPL.JUnit_ElevatorUserFactory#test_makeIntelligentElevatorUser_KO3()} </li>
	 * <li> {@link unit_tests.factoriesSPL.JUnit_ElevatorUserFactory#test_makeIntelligentElevatorUser_KO4()} </li>
	 * <li> {@link unit_tests.factoriesSPL.JUnit_ElevatorUserFactory#test_makePressAllButtonsElevatorUser()} </li>
	 * </ul>
	 * @param user_type - is the type of elevator user
	 * @param startFloor - the floor from which the user wishes to take the elevator
	 * @param endFloor - the floor to which the user wishes to travel using the elevator
	 * @param elevator - the elevator that the user wishes to travel on
	 * @throws IllegalArgumentException when the elevator is <code> null</code> 
	 * @throws IllegalArgumentException when the startFloor is out of bounds
	 * @throws IllegalArgumentException when the endFloor is out of bounds
	 * @throws IllegalArgumentException when the endFloor and startFloor are the same
	 * @return the elevator user constructed by the factory
	 */
	public  ElevatorUserSpecification makeElevatorUser(UserType user_type, int startFloor, int endFloor, ElevatorForUsersSpecification elevator)
	       throws IllegalArgumentException{
		
		if (elevator == null) 
			throw new IllegalArgumentException("You cannot assign a user to a null elevator");
		if (startFloor<0 || startFloor > elevator.topFloor()) 
			throw new IllegalArgumentException("start floor is out of bounds");
		if (endFloor<0 || endFloor > elevator.topFloor()) 
			throw new IllegalArgumentException("end floor is out of bounds");
		if (endFloor == startFloor) 
			throw new IllegalArgumentException("start and end floors must be different");
		
		if (user_type == UserType.INTELLIGENT) 
			return new IntelligentElevatorUser(startFloor, endFloor,  elevator);
		else if (user_type == UserType.INTELLIGENT_WITH_KEY) 
			return new IntelligentElevatorUserWithKey(startFloor, endFloor,  elevator);
		else return new PressAllButtonsElevatorUser(startFloor,  endFloor,  elevator);
	}
	
	/**
	 * Elevator user  of specified type is constructed with randomly generated valid route between floors (from -&gt; to) 
	 * @param user_type is the type of user
	 * @param rng is the random number generator used to construct the random floor values
	 * @param elevator is the elevator that the user wishes to travel on
	 * @throws IllegalArgumentException when the elevator is <code> null</code>
	 * @throws IllegalArgumentException when the random number generator (rng) is <code> null</code>
	 * @return the elevator user constructed by the factory
	 */
	public  ElevatorUserSpecification makeElevatorUser(UserType user_type, Random rng, ElevatorForUsersSpecification elevator)
           throws IllegalArgumentException{
		
		if (elevator == null) 
			throw new IllegalArgumentException("You cannot assign a user to a null elevator");
		
		if (rng == null) 
			throw new IllegalArgumentException("You cannot randomize using a null random number generator");
		
		if (user_type == UserType.INTELLIGENT) 
			return new IntelligentElevatorUser(rng,  elevator);
		else if (user_type == UserType.INTELLIGENT_WITH_KEY) 
			return new IntelligentElevatorUserWithKey(rng,  elevator);
		else return new PressAllButtonsElevatorUser(rng,  elevator);
	}
	
	/**
	 * Elevator user of random type is constructed  with randomly generated valid route between floors (from -&gt; to) 
	 * @param rng is the random number generator used to construct the random user type and the random floor values
	 * @param elevator is the elevator that the user wishes to travel on
	 * @throws IllegalArgumentException when the elevator is <code> null</code>
	 * @throws IllegalArgumentException when the random number generator (rng) is <code> null</code>
	 * @return the elevator user constructed by the factory
	 */
     public  ElevatorUserSpecification makeElevatorUser(Random rng, ElevatorForUsersSpecification elevator)
		
    	 throws IllegalArgumentException{
    			
    			if (elevator == null) 
    				throw new IllegalArgumentException("You cannot assign a user to a null elevator");
    			
    			if (rng == null) 
    				throw new IllegalArgumentException("You cannot randomize using a null random number generator");
    	
    			int user_index = rng.nextInt(UserType.values().length);
    			UserType user_type = UserType.values()[user_index];
    			
    			switch (user_type) {
    		    case INTELLIGENT:      
    		    	return new IntelligentElevatorUser(rng,  elevator);
				case PRESS_ALL_BUTTONS:      
    		    	return new PressAllButtonsElevatorUser(rng,  elevator);
    		    case INTELLIGENT_WITH_KEY:      
    		    	return new IntelligentElevatorUserWithKey(rng,  elevator);
         	    default: // should not be required
    		    	return new IntelligentElevatorUser(rng,  elevator);
    			 }
	}


}
