package interfaces;

import tools.HasInvariant;


/**
 * The Elevator code was developed as a sample solution to an educational OO design problem
 * <br>
 * The elevator must offer 2 interfaces: 
 * <ul>
 * <li> To the logic unit (specified by {@link ElevatorForLogicUnitSpecification})</li>
 * <li> To the users (specified by {@link ElevatorForUsersSpecification})</li>
 * </ul>
 * It also provides a means of updating the floor periodically for movement as decided by a logic unit.<br>
 * The elevator is observable.<br>
 * An inner thread (which can be turned on/off) can be used to update the state of the elevator periodically.
 * @author J Paul Gibson
 */
public interface ElevatorSpecification extends ElevatorForLogicUnitSpecification, ElevatorForUsersSpecification, HasInvariant{

	
	/**
	 * It is important that this maximum is <em> not </em> increased because the graphical view
	 * as specified in {@link views.JPanel_ElevatorSpecification} scales best to this number.<br>
	 * Further, the controller interface as specified in {@link controllers.KeyboardButtonsForElevator} is designed
	 * to require only a single keystroke (0..9) to specify any valid floor. <br>
	 */
	public static final int MAXIMUM_NUMBER_OF_FLOORS = 10;
	
	/**
	 * An elevator must have at least 2 floors to serve
	 */
	public static final int MINIMUM_NUMBER_OF_FLOORS = 2;

	/**
	 * Connect the observer to the elevator
	 * @param observer is the object that which will observe (and react) to elevator state updates
	 * @throws IllegalArgumentException if the specified observer is null
	 */
	public void connect_Observer(ElevatorObserverSpecification  observer) throws IllegalArgumentException;
	
	/**
	 * Turn on the logic unit thread of the elevator so that it moves periodically
	 */
	public void elevator_logic_unit_on();
	
	/**
	 * Turn off the logic unit thread of the elevator 
	 */
	public void elevator_logic_unit_off();

	
	/**
	 * move to next floor (as decided by the logic unit)
	 * @return if the lift should stop at the next floor (as decided by the logic unit)
	 */
	public  boolean updateFloor ();
	
	/**
	 * 
	 * @return if the lift is stopping at next floor (without updating the floor)
	 */
	public boolean stoppingAtNextFloor();
	
	/**
	 * 
	 * @return The logic unit type associated to the elevator
	 */
	public String  get_logic_unit_Type();
	
	
	/**
	 * 1) An elevator must have at least {@link #MINIMUM_NUMBER_OF_FLOORS} floors<br>
	 * 2) An elevator must have at most {@link #MAXIMUM_NUMBER_OF_FLOORS} floors<br>
	 * 3) The current floor of the elevator must be between 0 and the {@link #topFloor()}, inclusive<br>
	 * @return <code> true </code> if all these conditions are met, and <code> false </code> otherwise
	 */
	public  boolean invariant();

}
