package models.elevatorDoors;

import enumerations.DoorState;
import tools.HasInvariant;
import tools.InvariantBroken;
import unit_tests.doors.JUnit_DoorsOnFloors;

/**
 * Tested by {@link unit_tests.doors.JUnit_DoorsOnFloors} and {@link validation_tests.Validation_DoorsOnFloors}<br>
 * For managing the state of the doors at each of the floors of an elevator
 * (see {@link interfaces.ElevatorSpecification})
 * @author jpaulgibson
 */
public class DoorsOnFloors implements HasInvariant{
	
	/**
	 * The number of floors being served by the elevator is in range <code> 2 .. NUMBER_OF_FLOORS</code>
	 */
	public final int NUMBER_OF_FLOORS;
	
	/**
	 * The up requests on each floor<br>
	 * There must not be an up request on the top floor<br>
	 */
	private final DoorState [] doors;
	
	
	/**
	 * Initialises all doors to be closed
	 * @param numberOfFloors is the number of floors in the lift system
	 * @throws IllegalArgumentException if the number of floors is smaller than 2
	 * @throws InvariantBroken if not initialised in a safe state
	 */
	public DoorsOnFloors (int numberOfFloors) 
	       throws IllegalArgumentException, InvariantBroken{
			
		if (numberOfFloors <2) 
			throw new IllegalArgumentException("The number of floors must be greater than 1");
		
		NUMBER_OF_FLOORS = numberOfFloors;	
		
		doors = new DoorState [NUMBER_OF_FLOORS];

		
		for (int floor =0; floor<NUMBER_OF_FLOORS;floor++){
			doors [floor] = DoorState.CLOSED;
		}
		
		if (!invariant()) 
			throw new InvariantBroken("The doors have not been initialised in a safe state");
	}
	
	/**
	 * The following safety conditions are required:
	 * <ul>
	 * <li> number of floors must be at least 2 </li>
	 * <li> the number of doors is the same as the number of floors</li>
	 * <li> the doors cannot be open on more than 1 floor</li>
	 * </ul>
	 * Tested by {@link JUnit_DoorsOnFloors#test_invariant()}
	 * @return true if all these safety conditions are met, and false otherwise. 
	 */
	public boolean invariant(){
		
		if (NUMBER_OF_FLOORS<2) return false;
		if (doors.length != NUMBER_OF_FLOORS) return false;
		
		int openCount = 0;
		for (int floor =0; floor < NUMBER_OF_FLOORS; floor++)
			if (doors[floor]== DoorState.OPEN) openCount++;
		
		if (openCount>1) return false;
		
		return true;
	}
	
	/**
	 * Tested by {@link JUnit_DoorsOnFloors#test_isDoorOpen()}
	 * @param floor specifies the floor at which the door is being checked
	 * @throws IllegalArgumentException if the floor specified is not valid
	 * @return if a door is open at the specified floor
	 */
	public boolean isDoorOpen (int floor) throws IllegalArgumentException{
			
		if (floor<0 || floor > NUMBER_OF_FLOORS-1) 
			throw new IllegalArgumentException("The floor specified is not valid");
		
		return (doors[floor] == DoorState.OPEN);
		}
	
	
	/**
	 * Open the specified door<br>
	 * @param floor specifies the floor at which the door is to be opened
	 * @throws IllegalArgumentException if the floor specified is not valid
	 * @throws IllegalStateException if a door is already open at a different floor
	 * @throws InvariantBroken if the doors are put into an unsafe state
	 */
	public void openDoor(int floor)throws IllegalArgumentException, IllegalStateException, InvariantBroken{
			
		if (floor<0 || floor >= NUMBER_OF_FLOORS) 
			throw new IllegalArgumentException("The floor specified is not valid");
		
		
		for (int fl =0; fl< NUMBER_OF_FLOORS; fl++)
			if (floor != fl && doors[fl]== DoorState.OPEN) 
				throw new IllegalStateException("A set of doors is already  open at another floor");
		
		else doors[floor] = DoorState.OPEN;
		
		// This should never happen
		if (!invariant()) 
			throw new InvariantBroken("The doors are in an unsafe state");
		}
	
	/**
	 * Close the specified door<br>
	 * Tested by {@link JUnit_DoorsOnFloors#test_closeDoor()}
	 * @param floor specifies the floor at which the door is to be opened
	 * @throws IllegalArgumentException if the floor specified is not valid
	 */
	public void closeDoor(int floor)throws IllegalArgumentException, InvariantBroken{
			
		if (floor<0 || floor >= NUMBER_OF_FLOORS) 
			throw new IllegalArgumentException("The floor specified is not valid");
		
		else doors[floor] = DoorState.CLOSED;
		
		// This should never happen
		if (!invariant()) 
			throw new InvariantBroken("The doors are in an unsafe state");
		}
	
	
	/**
	 * Tested by {@link JUnit_DoorsOnFloors#test_toString()} <br>
	 * Typical string representation is of the form:
	 * <pre>
	 * DoorState States For Elevator At Floor: 0- 1-Open 2- 3- 4- 5- 6- 7- 8- 9-
	 * </pre>
	 * In this example, there are 10 floors and the doors on floor 1 are Open
	 */
	public String toString (){
		
		String str = "DoorState States For Elevator At Floor:";
			for (int floor=0; floor< NUMBER_OF_FLOORS;floor++){
				str = str + " "+floor+"-";
				if (doors[floor]==DoorState.OPEN) str = str+"OPEN";
				}
				
			return str;
	}
	
	
}

