package models.elevators;

import java.util.Vector;

import tools.Ticker;
import tools.HasInvariant;
import tools.TickerObserverSpecification;
import enumerations.DirectionState;
import models.elevatorButtons.DirectionRequestButtonsOnFloors;
import models.elevatorButtons.FloorButtonsInElevator;
import interfaces.ElevatorObserverSpecification;
import interfaces.ElevatorSpecification;
import interfaces.LogicUnitForElevatorSpecification;

/**
 * Implements the {@link ElevatorSpecification}.<br>
 * Uses an internal {@link Ticker} to update the elevator state (movements) at a regular frequency
 * @author jpaulgibson
 */
public class Elevator implements ElevatorSpecification, TickerObserverSpecification, HasInvariant{
	
	
	 /** 
	  * The time (in ms) between ticks of the internal timer that launches updates to the elevator state
	  * as it moves between floors
	  */
	 public final int TICK_TIME;
	 
	 /**
	  * If no tick time is specified then this is the default value;
	  */
	 final int DEFAULT_TICK_TIME = 1000;
	
	  /**
	   * The ticker that is used to add new users to the elevator system
	   */
	  final Ticker TICKER_FOR_UPDATING_ELEVATOR_STATE;
	
    /**
     * All the observers of the elevator.<br>
     * We expect the elevator updates to be observed by:
     * <ul>
     * <li> users - through the {@link simulations.Users_Simulation} </li> 
     * <li> the graphical view - through the {@link views.JPanel_ElevatorSpecification} </li>
     * </ul>
     */
    final  Vector<ElevatorObserverSpecification> observers;

	/**
	 * Whether the logic_unit regularly updates the state of the elevator every {@link #TICK_TIME} ms.
	 */
	boolean logic_unit_on;


	
	/**
	 * Represents the number of floors that the elevator serves during its operation
	 * Must be no smaller than 2.
	 */
	public final int NUMBER_OF_FLOORS;
	
	/**
	 * The current direction in which the elevator is travelling
	 */
	protected DirectionState currentDirection; 
	
	/**
	 * Will the elevator stop at the next floor?
	 */
	protected boolean stoppingAtNextFloor;
	
	/**
	 * The floor to which the elevator is currently targeted
	 */
	protected int currentFloor;
	
	/**
	 * The buttons at each floor which permit requests from users to go up or down
	 */
	private DirectionRequestButtonsOnFloors  directionButtons;
	
	/**
	 * The buttons in the elevator which permit users to request to go to a specific floor
	 */
	protected FloorButtonsInElevator  floorButtons;

	/**
	 * The component that provides the logic for controlling the movement of the elevator
	 */
	private LogicUnitForElevatorSpecification logic_unit = null;
	
	
	public Elevator (int numberOfFloors) throws IllegalArgumentException{
		
		if (numberOfFloors <2) 
			throw new IllegalArgumentException("The number of floors must be greater than 2");
		NUMBER_OF_FLOORS = numberOfFloors;
		
		currentFloor =0;
		stoppingAtNextFloor = false;
		currentDirection = DirectionState.STAY;
		directionButtons = new DirectionRequestButtonsOnFloors(NUMBER_OF_FLOORS);
		floorButtons = new FloorButtonsInElevator(NUMBER_OF_FLOORS);
		
		TICK_TIME = DEFAULT_TICK_TIME;
		logic_unit_on = false;
		TICKER_FOR_UPDATING_ELEVATOR_STATE = new Ticker(TICK_TIME);
		TICKER_FOR_UPDATING_ELEVATOR_STATE.connect_Observer(this);
		this.observers = new Vector<ElevatorObserverSpecification>();

	}
	
	
	public Elevator (int numberOfFloors, int delay_time) throws IllegalArgumentException{
			
		if (numberOfFloors <2) 
			throw new IllegalArgumentException("The number of floors must be greater than 2");
		NUMBER_OF_FLOORS = numberOfFloors;
		
		currentFloor =0;
		stoppingAtNextFloor = false;
		currentDirection = DirectionState.STAY;
		directionButtons = new DirectionRequestButtonsOnFloors(NUMBER_OF_FLOORS);
		floorButtons = new FloorButtonsInElevator(NUMBER_OF_FLOORS);
		
		logic_unit_on = false;
		TICK_TIME = delay_time;
		TICKER_FOR_UPDATING_ELEVATOR_STATE = new Ticker(TICK_TIME);
		TICKER_FOR_UPDATING_ELEVATOR_STATE.connect_Observer(this);
		this.observers = new Vector<ElevatorObserverSpecification>();

	}
	
	@Override
	public void connect_Observer(ElevatorObserverSpecification observer) throws IllegalArgumentException {
		
		if (observer == null) throw new IllegalArgumentException("Cannot connect a null observer");
		observers.add(observer);
	}

	@Override
	public DirectionState currentDirection(){ return currentDirection;}
	
	@Override
	public boolean stoppingAtNextFloor(){ return stoppingAtNextFloor;}
	
	@Override
	public int currentFloor(){ return currentFloor;}
	
	@Override
	public int topFloor(){return NUMBER_OF_FLOORS-1;}
	
	@Override
	public boolean updateFloor (){
		
	    //	move to new floor
		
		if (currentDirection == DirectionState.UP && currentFloor < topFloor()){
			

			if (stoppingAtNextFloor) directionButtons.cancelUpRequest(currentFloor); 
			currentFloor++; 

		}
		else if (currentDirection == DirectionState.DOWN && currentFloor > 0){
			

			if (stoppingAtNextFloor) directionButtons.cancelDownRequest(currentFloor); 
			currentFloor--; 

		}
		
		// calculate new direction
		currentDirection = logic_unit.calculateDirection();
		
		stoppingAtNextFloor = logic_unit.stopAtNextFloor();
		
		// if we stop at the new floor then cancel the appropriate floor requests
		if (stoppingAtNextFloor){
			
			if (currentDirection == DirectionState.UP) directionButtons.cancelUpRequest(currentFloor);
			if (currentDirection == DirectionState.DOWN) directionButtons.cancelDownRequest(currentFloor);
			floorButtons.cancelFloorRequestInElevator(currentFloor);
		}
		

		for (ElevatorObserverSpecification ob: observers)
			ob.elevatorUpdated();

		
		return stoppingAtNextFloor;
	}

	@Override
	public boolean downPressedAtFloor(int floor) throws IllegalArgumentException {
		return directionButtons.downPressedAtFloor(floor);
	}

	@Override
	public boolean upPressedAtFloor(int floor) throws IllegalArgumentException {
		return directionButtons.upPressedAtFloor(floor);
	}

	@Override
	public boolean floorPressedInElevator(int floor) throws IllegalArgumentException {
		
		return floorButtons.floorPressedInElevator(floor);
	}

	@Override
	public boolean noMoreRequestsBelow(int floor) throws IllegalArgumentException {

		return (directionButtons.noMoreRequestsBelow(floor) &&
				floorButtons.noMoreRequestsBelow(floor));
	}


	@Override
	public boolean noMoreRequestsAbove(int floor) throws IllegalArgumentException {
		return (directionButtons.noMoreRequestsAbove(floor) &&
				floorButtons.noMoreRequestsAbove(floor));
	}

	@Override
	public void pressUp(int floor) throws IllegalArgumentException {
		
		directionButtons.pressUp(floor);	
	}

	@Override
	public void pressDown(int floor) throws IllegalArgumentException {

		directionButtons.pressDown(floor);	
	}

	@Override
	public void pressFloorInElevator(int floor) throws IllegalArgumentException {
		floorButtons.pressFloorInElevator(floor);
		
	}

	@Override
	public void installLogicUnit(LogicUnitForElevatorSpecification logic_unit)	throws IllegalArgumentException {

		if (logic_unit == null) 
			throw new IllegalArgumentException("The logic unit component must not be null");
		if (this.logic_unit != logic_unit){
			this.logic_unit = logic_unit;
			logic_unit.bindToElevator(this);
		}
		
	}
	
	@Override
	public void elevator_logic_unit_on() {
		
		 logic_unit_on = true;
		 TICKER_FOR_UPDATING_ELEVATOR_STATE.animation_on();
		
	}


	@Override
	public void elevator_logic_unit_off() {
		
		
		logic_unit_on = false;
		TICKER_FOR_UPDATING_ELEVATOR_STATE.animation_off();
		
	}

	public void observe_ticker() {
		
		updateFloor();
		
	}



   public String toString(){
	   
	   String str ="\nElevator State:\n";
	     
	   str = str+ "DirectionState = ";
	   if (currentDirection == DirectionState.UP) str = str+ "UP";
	   if (currentDirection == DirectionState.DOWN) str = str+ "DOWN";
	   if (currentDirection == DirectionState.STAY) str = str+ "STAY";
	   
	   str = str+ "\nStopping at floor = "+ stoppingAtNextFloor;
	   
	   str = str+"\nFloor = "+ currentFloor;
	   str = str+"\nNumber of floors = "+NUMBER_OF_FLOORS;
	   
	   str = str+ "\n"+floorButtons;
	   str = str+ "\n"+directionButtons;
	   
	   str = str +"\nLogic unit type = "+logic_unit;
	   
	   if (logic_unit_on) str = str+" is on"; else str = str+" is off";
	   
	   str = str +"\nDelay time = "+TICK_TIME;
	   
	   if (!invariant()) str = str+ "\n **** WARNING - Invariant Broken ***** \n";
	   
	   return str;
   }

  /**
   * As well as checking the requirements as specified in {@link interfaces.ElevatorSpecification#invariant()}, it 
   * also checks that:
   * <ul>
   * <li> the direction buttons components is in a safe state </li>
   * <li> the floor buttons component is in a safe state </li>
   * <li> the number of buttons is coherent with the number of floors </li>
   * </ul>
   */
  @Override
  public boolean invariant() {

	if (NUMBER_OF_FLOORS < MINIMUM_NUMBER_OF_FLOORS) return false;
	if (NUMBER_OF_FLOORS > MAXIMUM_NUMBER_OF_FLOORS) return false;
	if (currentFloor<0 || currentFloor>= NUMBER_OF_FLOORS ) return false;
	
	if (!directionButtons.invariant()) return false;
	if (!floorButtons.invariant()) return false;
	
	if(directionButtons.NUMBER_OF_FLOORS != NUMBER_OF_FLOORS) return false;
	if(floorButtons.NUMBER_OF_FLOORS != NUMBER_OF_FLOORS) return false;
	
	return true;
  }

@Override
public String get_logic_unit_Type() {
	
	return logic_unit.toString();
}

}
