package tools;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.Timer;

/**
 * A ticker thread that generates an observable event every {@link #TICK_TIME} milliseconds
 * Any object that wishes to observe the ticker must implement the  {@link tools.TickerObserverSpecification}<br>
 * @see <a href="https://en.wikipedia.org/wiki/Observer_pattern"> wikipedia - observer design pattern</a>
 * 
 * @author jpaulgibson
 *
 */
public class Ticker implements ActionListener{
	
	/** 
	 * The time (in ms) between ticks of the  timer that can be observed
	 */
	final int TICK_TIME;

	/**
	 * If the ticker is {@link #animated} then all observers are informed when each tick action occurs
	 */
    public void actionPerformed(ActionEvent e) {

    	
    if (animated)	
    	 for (TickerObserverSpecification observer: observers)
 		    observer.observe_ticker();  
    }
     

   /**
    * Swing Timer used for the periodic event generation
    */
    Timer TIMER;

    /**
     * The observers that will be updated every time a tick action occurs, provided the ticker is {@link #animated}
     */
    final  Vector<TickerObserverSpecification> observers;

    /**
     * A switch for turning on-off the ticker animation
     */
    boolean animated;
	
    /**
     * The ticker listens for the {@link #TIMER} actions.<br>
     * Initially, there are no observers of the ticker, 
     * these can be added later using {@link #connect_Observer(TickerObserverSpecification)},
     * and the ticker is not {@link #animated}.
     * @param tick_time is the time in milliseconds between actions generated by the {@link #TIMER}
     */
	public Ticker (int tick_time){

		TICK_TIME = tick_time;
		this.TIMER = new Timer(TICK_TIME, this); // TODO
		animated = false;
		this.observers = new Vector<TickerObserverSpecification>();
	}

    /**
     * 
     * @param observer which will subsequently be informed of every tick whilst the ticker is {@link #animated}
     */
	public void connect_Observer(TickerObserverSpecification observer) {
		observers.add(observer);
	}
	
	/**
	 * Turn on the animation - so tick updates are observed 
	 */
	public void animation_on() {
		animated = true;
		TIMER.start();	
	}

	/**
	 * Turn off the animation - so tick updates are not observed
	 */
	public void animation_off() {
		TIMER.stop();
		animated = false;
	}
	
}
