package unit_tests.elevatorUsers.unitTestAbstractions_elevatorUsers;

import java.util.Random;

import interfaces.ElevatorSpecification;
import interfaces.ElevatorUserSpecification;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import enumerations.UserState;
import factoriesSPL.ElevatorFactory;

/** 
 * Unit tests for {@link interfaces.ElevatorUserSpecification}.<br>
 * In order to instantiate an elevator user for test, we are required to construct an elevator.<br>
 * The choice of elevator is completely arbitrary, as we make only the assumption that the user
 * wishes to take the elevator from floor 1 to floor 2.<br>
 * @author jpaulgibson
 */
public abstract class JUnit_ElevatorUserSpecification {
	
	static Random rng = new Random();
	
	protected static final int USER_FROM_FLOOR = 1;
	protected static final int USER_TO_FLOOR = 2;
	
	protected static final ElevatorSpecification elevator = ElevatorFactory.makeElevator();
	
	protected ElevatorUserSpecification elevatorUserUnderTest;
	
	/**
	 * Construct an <code>elevatorUserUnderTest</code>  object for a user on the bottom floor (0) who wishes 
	 * to go up a single floor (to floor 1). These values are chosen because all elevators are
	 * guaranteed by the class invariant to have at least 2 floors (starting from 0).
	 * (See {@link interfaces.ElevatorSpecification#invariant()}).<br><br>
	 * This is an abstract method because we do not know the concrete class of the <code>elevatorUserUnderTest</code> t object
	 * and so defer its construction to any concrete JUnit test class which  extends this one.<br><br>
	 * Tested by {@link #test_SetUp()}.
	 */
	@Before
	public  abstract void setUp();

	
	@After
	public void tearDown() {
		elevatorUserUnderTest = null;
	}
	
	/**
	 * Iterate a set number of times (specified by NUMBER_OF_CONSTRUCTION_TESTS) and call the 
	 * constructor operator of the elevatorUserUnderTest<br>
	 * Check that the ID counter gets incremented each time.<br>
	 * This is an abstract method because we do not know the concrete class of the 
	 * <code>elevatorUserUnderTest</code>  object
	 * and so defer its construction to any concrete JUnit test class which  extends this one.<br><br>
	 */
	@Test
	public abstract void test_constructionIDCounter();
	
	
	/**
	 * Construct 2 different user objects and check that they have different hash values<br>
	 * Check that the equals method works for all possible input value:
	 * <ul>
	 * <li> null </li>
	 * <li> an object which is not an elevator user </li>
	 * <li> the same object </li>
	 * <li> a different object </li>
	 * </ul>
	 * This is an abstract method because we do not know the concrete class of the 
	 * <code>elevatorUserUnderTest</code>  objects
	 * and so defer their construction to any concrete JUnit test class which  extends this one.<br><br>
	 */
	@Test
	public abstract void test_equalsHash();
	
	/**
	 * Check that an exception is thrown when a user wants to go to the floor where they are already located.<br>
	 * This is an abstract method because we do not know the concrete class of the 
	 * <code>elevatorUserUnderTest</code>  object
	 * and so defer its construction to any concrete JUnit test class which  extends this one.<br><br>
	 */
	@Test(expected=IllegalArgumentException.class)
	public abstract void test_constructionKO1();
	
	/**
	 * Check that an exception is thrown when a user is constructed to be waiting at a floor below the bottom.<br>
	 * This is an abstract method because we do not know the concrete class of the 
	 * <code>elevatorUserUnderTest</code>  object
	 * and so defer its construction to any concrete JUnit test class which  extends this one.<br><br>
	 */
	@Test(expected=IllegalArgumentException.class)
	public abstract void test_constructionKO2();
	
	/**
	 * Check that an exception is thrown when a user is constructed who wishes to go to a floor below the bottom.<br>
	 * This is an abstract method because we do not know the concrete class of the 
	 * <code>elevatorUserUnderTest</code>  object
	 * and so defer its construction to any concrete JUnit test class which  extends this one.<br><br>
	 */
	@Test(expected=IllegalArgumentException.class)
	public abstract void test_constructionKO3();
	
	/**
	 * Initially, after {@link #setUp()}, the  test case must be a user
	 * waiting for an elevator on the bottom floor (0) in order 
	 * to go up a single floor (to floor 1)<br>
	 */
	@Test
	public void test_SetUp(){
		
		Assert.assertEquals(UserState.ARRIVED_AT_FLOOR,elevatorUserUnderTest.get_state());	
		Assert.assertEquals(USER_FROM_FLOOR ,elevatorUserUnderTest.get_from());
		Assert.assertEquals(USER_TO_FLOOR ,elevatorUserUnderTest.get_to());	
		
		Assert.assertTrue(elevatorUserUnderTest.invariant());
	}
	
	/**
	 * Unit test for {@link ElevatorUserSpecification#pickedUpAtFloor()} and
	 * {@link ElevatorUserSpecification#deliveredToFloor()}.<br>
	 * Checks the valid state sequence ARRIVED_AT_FLOOR -pressedButtonAtFLoor- WAITING_AT_FLOOR -pickedUpAtFloor- IN_ELEVATOR -deliveredToFloor- SERVICED
	 */
	@Test
	public void test_stateTransitions(){
		
		Assert.assertEquals(UserState.ARRIVED_AT_FLOOR,elevatorUserUnderTest.get_state());
		elevatorUserUnderTest.pressedButtonAtFloor();
		Assert.assertEquals(UserState.WAITING_AT_FLOOR,elevatorUserUnderTest.get_state());
		elevatorUserUnderTest.pickedUpAtFloor();
		Assert.assertEquals(UserState.IN_ELEVATOR,elevatorUserUnderTest.get_state());	
		elevatorUserUnderTest.deliveredToFloor();
		Assert.assertEquals(UserState.SERVICED,elevatorUserUnderTest.get_state());
		
		Assert.assertTrue(elevatorUserUnderTest.invariant());
	}
	
	/**
	 * Unit test for {@link ElevatorUserSpecification#deliveredToFloor()}.<br>
	 * Checks the invalid state transition WAITING_AT_FLOOR -deliveredToFloor- 
	 */
	@Test(expected=IllegalStateException.class)
	public void test_stateTransitionsKO1(){	
		
		elevatorUserUnderTest.pressedButtonAtFloor();
		Assert.assertEquals(UserState.WAITING_AT_FLOOR,elevatorUserUnderTest.get_state());		
		elevatorUserUnderTest.deliveredToFloor();
		
	}

	/**
	 * Unit test for {@link ElevatorUserSpecification#deliveredToFloor()} and
	 * {@link ElevatorUserSpecification#pickedUpAtFloor()}.<br>
	 * Checks the invalid state transition IN_ELEVATOR -pickedUpAtFloor- 
	 */
	@Test(expected=IllegalStateException.class)
	public void test_stateTransitionsKO2(){	
		
		elevatorUserUnderTest.pickedUpAtFloor();
		Assert.assertEquals(UserState.IN_ELEVATOR,elevatorUserUnderTest.get_state());
		elevatorUserUnderTest.pickedUpAtFloor();

    }
	
	/**
	 * Unit test for {@link ElevatorUserSpecification#deliveredToFloor()}
	 * and {@link ElevatorUserSpecification#pickedUpAtFloor()}.<br>
	 * Checks the invalid state transition SERVICED -deliveredToFloor- 
	 */
	@Test(expected=IllegalStateException.class)
	public void test_stateTransitionsKO3(){
		
		elevatorUserUnderTest.pickedUpAtFloor();
		elevatorUserUnderTest.deliveredToFloor();	
		Assert.assertEquals(UserState.SERVICED,elevatorUserUnderTest.get_state());
		elevatorUserUnderTest.deliveredToFloor();
	}
	
	/**
	 * Unit test for {@link ElevatorUserSpecification#deliveredToFloor()}
	 * and {@link ElevatorUserSpecification#pickedUpAtFloor()}.<br>
	 * Checks the invalid state transition SERVICED -pickedUpAtFloor- 
	 */
	@Test(expected=IllegalStateException.class)
	public void test_stateTransitionsKO4(){
		

		elevatorUserUnderTest.pickedUpAtFloor();
		elevatorUserUnderTest.deliveredToFloor();
		Assert.assertEquals(UserState.SERVICED,elevatorUserUnderTest.get_state());
		elevatorUserUnderTest.pickedUpAtFloor();
	}
	
	/**
	 * Unit test for {@link ElevatorUserSpecification#toString()}
	 */
	@Test
	public abstract void test_toString();
	
	/**
	 * Unit test for {@link ElevatorUserSpecification#makeDecision()}
	 */
	@Test
	public abstract void test_makeDecision();
}
