package unit_tests.factoriesSPL;

import interfaces.ElevatorSpecification;
import enumerations.SecurityType;

import org.junit.Assert;
import org.junit.Test;

import enumerations.LogicType;
import factoriesSPL.ElevatorFactory;

/**
 * Unit tests for constructing a range of different elevator configurations as permitted by the
 * {@link factoriesSPL.ElevatorFactory}<br><br>
 * We test  that the elevators are constructed in an initially safe state that respects the invariant
 * as defined by {@link ElevatorSpecification#invariant()}.
 * @author jpaulgibson
 * @see validation_tests.validation_factoriesSPL.Validation_ElevatorFactory
 */
public class JUnit_ElevatorFactory {
	

	/**
	 * Unit test for {@link ElevatorFactory#makeElevator( ) }<br>
	 * Tests that the factory builds an elevator with the default values in a safe state
	 */
	@Test
	public void test_makeElevator_default(){
		
		ElevatorSpecification elevator = ElevatorFactory.makeElevator();
		Assert.assertTrue(elevator.invariant());
	}
	
	/**
	 * Unit test for {@link ElevatorFactory#makeElevator(Random ) }<br>
	 * Tests that the factory builds an elevator with the default values in a safe state
	 */
	@Test
	public void test_makeElevator_random(){
		
		int NUMBER_OF_RANDOM_GENERATIONS = 1000;
		for (int test_count = 0; test_count < NUMBER_OF_RANDOM_GENERATIONS; test_count++){
			  ElevatorSpecification elevator = ElevatorFactory.makeElevator();
		      Assert.assertTrue(elevator.invariant());
		}
	}
	
	/**
	 * Unit test for {@link ElevatorFactory#makeElevator(boolean, int, LogicType, SecurityType, int) }<br>
	 * Tests that the factory builds an elevator with the given values in a safe state.<br>
	 * We test only a single possible configuration as the random tests also test multiple configurations
	 */
	@Test
	public void test_makeElevator(){
		

			  ElevatorSpecification elevator = 
					  ElevatorFactory.makeElevator(false, 2, LogicType.STUPID, SecurityType.NONE, 1000);
			  
		      Assert.assertTrue(elevator.invariant());
	}
	
	/**
	 * Unit test for {@link ElevatorFactory#makeElevator(boolean, int, LogicType, SecurityType, int) }<br>
	 * Tests the exception generated when there are not enough floors
	 */
	@Test(expected = IllegalArgumentException.class)
	public void test_makeElevatorException1(){
		
		 @SuppressWarnings("unused")
			  ElevatorSpecification elevator = 
					  ElevatorFactory.makeElevator(false, ElevatorFactory.MINIMUM_NUMBER_OF_FLOORS-1, LogicType.STUPID, SecurityType.NONE, 1000);
	}
	
	/**
	 * Unit test for {@link ElevatorFactory#makeElevator(boolean, int, LogicType, SecurityType, int) }<br>
	 * Tests the exception generated when there are too many floors
	 */
	@Test(expected = IllegalArgumentException.class)
	public void test_makeElevatorException2(){	

			  @SuppressWarnings("unused")
			ElevatorSpecification elevator = 
					  ElevatorFactory.makeElevator(false, ElevatorFactory.MAXIMUM_NUMBER_OF_FLOORS+1, LogicType.STUPID, SecurityType.NONE, 1000);
	}
	
}
