package unit_tests.factoriesSPL;

import interfaces.ElevatorForUsersSpecification;
import interfaces.ElevatorUserSpecification;

import java.util.Random;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import enumerations.UserType;
import factoriesSPL.ElevatorFactory;
import factoriesSPL.ElevatorUserFactory;


/**
 * Simple unit test for constructing a range of different elevator user configurations as permitted by the
 * {@link factoriesSPL.ElevatorUserFactory}<br><br>
 * We test only that they are constructed in an initially safe state that respects the invariant.
 * @author jpaulgibson
 * @see validation_tests.validation_factoriesSPL.Validation_ElevatorUserFactory
 */
public class JUnit_ElevatorUserFactory {
	
	/**
	 * A dummy elevator to test the creation of a specified user
	 */
	private final static ElevatorForUsersSpecification ELEVATOR = ElevatorFactory.makeElevator();
	
	
	/**
	 * A RNG for the random factory creation
	 */
	private final static Random RNG = new Random();
	
	private  ElevatorUserFactory factoryUnderTest = null;
	
	@Before
	public void set_up(){
		factoryUnderTest = ElevatorUserFactory.getInstance();
	}
	
	@After
	public void tear_down(){
		factoryUnderTest = null;
	}
	
	/**
	 * Unit test for {@link factoriesSPL.ElevatorUserFactory#makeElevatorUser() }<br>
	 * Tests that the factory builds an elevator user that respects the invariant.
	 */
	@Test
	public void test_makeDefaultElevatorUser(){
		
		 ElevatorUserSpecification user = factoryUnderTest.makeElevatorUser();
		 Assert.assertTrue(user.invariant());
	} 
	
	
	
	/**
	 * Unit test for {@link factoriesSPL.ElevatorUserFactory#makeElevatorUser(Random rng, ElevatorForUsersSpecification elevator) }<br>
	 * Tests that the factory builds an elevator user that respects the invariant  <br>
	 * Does this by running the random constructor a specified number of times.
	 */
	@Test
	public void test_makeRandomElevatorUser_OK(){
		
		int NUMBER_OF_RANDOM_TESTS = 100;
		for (int testCount = 0; testCount<NUMBER_OF_RANDOM_TESTS; testCount++){
		 ElevatorUserSpecification user = factoryUnderTest.makeElevatorUser(RNG, ELEVATOR);
		 Assert.assertTrue(user.invariant());}
	}
	
	/**
	 * Unit test for {@link factoriesSPL.ElevatorUserFactory#makeElevatorUser(Random rng, ElevatorForUsersSpecification elevator) }<br>
	 * Tests that an exception is thrown when the random number generator parameter is null
	 */
	@Test(expected = IllegalArgumentException.class)
	public void test_makeRandomElevatorUser_KO1(){
		
		@SuppressWarnings("unused")
		ElevatorUserSpecification user = factoryUnderTest.makeElevatorUser(null, ELEVATOR);

	}
	
	/**
	 * Unit test for {@link factoriesSPL.ElevatorUserFactory#makeElevatorUser(Random rng, ElevatorForUsersSpecification elevator) }<br>
	 * Tests that an exception is thrown when the elevator parameter is null
	 */
	@Test(expected = IllegalArgumentException.class)
	public void test_makeRandomElevatorUser_KO2(){
		
		@SuppressWarnings("unused")
		ElevatorUserSpecification user = factoryUnderTest.makeElevatorUser(RNG, null);
	}

	
	/**
	 * Unit test for {@link factoriesSPL.ElevatorUserFactory#makeElevatorUser(UserType, int, int, ElevatorForUsersSpecification ) }<br>
	 * Tests that the factory builds an intelligent elevator user that respects the invariant  <br>
	 * Does this by running the constructor on all valid intelligent users.
	 */
	@Test
	public void test_makeIntelligentElevatorUser_OK(){
		
		
		for (int from = 0; from < ELEVATOR.topFloor(); from++)
			for (int to = 0; to < ELEVATOR.topFloor(); to++)
				if (from!=to){
		            ElevatorUserSpecification user = 
		            		factoryUnderTest.makeElevatorUser(UserType.INTELLIGENT, from, to, ELEVATOR);
		            Assert.assertTrue(user.invariant());}
	}
	
	/**
	 * Unit test for {@link factoriesSPL.ElevatorUserFactory#makeElevatorUser(UserType, int, int, ElevatorForUsersSpecification ) }<br>
     * Tests that an exception is thrown when the elevator parameter is null
	 */
	@Test(expected = IllegalArgumentException.class)
	public void test_makeIntelligentElevatorUser_KO1(){
		
		int FROM =0;
		int TO = 1;
		
		@SuppressWarnings("unused")
		ElevatorUserSpecification user = factoryUnderTest.makeElevatorUser(UserType.INTELLIGENT, FROM, TO, null);

	}
	
	/**
	 * Unit test for {@link factoriesSPL.ElevatorUserFactory#makeElevatorUser(UserType, int, int, ElevatorForUsersSpecification ) }<br>
     * Tests that an exception is thrown when the from floor is out of bounds
	 */
	@Test(expected = IllegalArgumentException.class)
	public void test_makeIntelligentElevatorUser_KO2(){
		
		int FROM =-1;
		int TO = 1;
		
		@SuppressWarnings("unused")
		ElevatorUserSpecification user = factoryUnderTest.makeElevatorUser(UserType.INTELLIGENT, FROM, TO, ELEVATOR);

	}
	
	/**
	 * Unit test for {@link factoriesSPL.ElevatorUserFactory#makeElevatorUser(UserType, int, int, ElevatorForUsersSpecification ) }<br>
     * Tests that an exception is thrown when the destination floor is out of bounds
	 */
	@Test(expected = IllegalArgumentException.class)
	public void test_makeIntelligentElevatorUser_KO3(){
		
		int FROM =0;
		int TO = ELEVATOR.topFloor()+1;
		
		@SuppressWarnings("unused")
		ElevatorUserSpecification user = factoryUnderTest.makeElevatorUser(UserType.INTELLIGENT, FROM, TO, ELEVATOR);

	}
	
	/**
	 * Unit test for {@link factoriesSPL.ElevatorUserFactory#makeElevatorUser(UserType, int, int, ElevatorForUsersSpecification ) }<br>
     * Tests that an exception is thrown when the to and from floors are the same
	 */
	@Test(expected = IllegalArgumentException.class)
	public void test_makeIntelligentElevatorUser_KO4(){
		
		int FROM =1;
		int TO = 1;
		
		@SuppressWarnings("unused")
		ElevatorUserSpecification user = factoryUnderTest.makeElevatorUser(UserType.INTELLIGENT, FROM, TO, ELEVATOR);

	}
	
	/**
	 * Unit test for {@link factoriesSPL.ElevatorUserFactory#makeElevatorUser(UserType, int, int ,ElevatorForUsersSpecification ) }<br>
	 * Tests that the factory builds an press all buttons elevator user that respects the invariant  <br>
	 * Does this by running the constructor on all valid press all button users.
	 */
	@Test
	public void test_makePressAllButtonsElevatorUser(){
		
		
		for (int from = 0; from < ELEVATOR.topFloor(); from++)
			for (int to = 0; to < ELEVATOR.topFloor(); to++)
				if (from!=to){
		            ElevatorUserSpecification user = 
		            		factoryUnderTest.makeElevatorUser(UserType.PRESS_ALL_BUTTONS, from, to, ELEVATOR);
		            Assert.assertTrue(user.invariant());}
	}

}
