
package validation_tests;

import interfaces.ElevatorSpecification;

import java.util.ArrayList;
import java.util.Random;

import tools.DateHeader;
import tools.SeedRNGCommandLine;
import models.elevatorUsers.PressAllButtonsElevatorUser;
import factoriesSPL.ElevatorFactory;

/**
 * Validation test for {@link PressAllButtonsElevatorUser}.<br>
 * The test is random (a seed value can be provided as a command line parameter).
 * <ul>
 * <li> Initially the elevator (created using the default constructor) has no user requests. </li>
 * <li> Iterate the creation of a random (press all buttons) user a set number of times ( specified by {@link #NUMBER_OF_RANDOM_USERS})<br>
 * During the iteration we construct only valid users. <br>
 * We check that all users have pressed all the buttons available/visible to them</li>
 * <li> Pick up a new (press all buttons) user on the bottom floor who wishes to go to the first floor.
 *    We check that when the user enters  the elevator then they press all the buttons inside the elevator
 * <li>   Deliver the user to the first floor.<br> We check that they exit the elevator and press all buttons on the floor.
 * <li> Finally, we check exception cases
 *    <ul>
 *    <li> try to construct an invalid user (who wishes to go to the floor on which they are currently located)</li>
 *    <li> try to construct an invalid user (who wished to use a null elevator) </li>
 *    </ul> </li>
 * </ul>
 * 
 * Typical output:
<pre>
The seed used for the random number generator in the test is 0.
You can override this value by passing an integer value as a main argument parameter, if you so wish.


********************************************************************
Execution Date/Time 2015/07/28 09:14:47
********************************************************************

Initially the elevator has no user requests

Elevator State:
DirectionState = STAY
Stopping at floor = false
Floor = 0
Number of floors = 4
Requested Floors Inside Elevator:
DirectionState Requests For Elevator At Floor: 0- 1- 2- 3-
Controller type = Smart Controller
DoorState States For Elevator At Floor: 0-OPEN 1- 2- 3-

Then we add random (press all buttons) users to floors who decide which buttons to press
PressAllButtonsElevatorUser 0 - from: 2 to: 3 currently WAITING_AT_FLOOR
PressAllButtonsElevatorUser 1 - from: 0 to: 2 currently WAITING_AT_FLOOR
PressAllButtonsElevatorUser 2 - from: 2 to: 1 currently WAITING_AT_FLOOR
PressAllButtonsElevatorUser 3 - from: 2 to: 0 currently WAITING_AT_FLOOR

Now the state of the elevator after these buttons have been pressed is:

Elevator State:
DirectionState = STAY
Stopping at floor = false
Floor = 0
Number of floors = 4
Requested Floors Inside Elevator:
DirectionState Requests For Elevator At Floor: 0-U 1- 2-UD 3-
Controller type = Smart Controller
DoorState States For Elevator At Floor: 0-OPEN 1- 2- 3-

Add a new  (press all buttons) user to go from floor 0 to floor 1

Check that the user enters the elevator and presses all buttons inside the elevator

Elevator State:
DirectionState = UP
Stopping at floor = true
Floor = 0
Number of floors = 4
Requested Floors Inside Elevator: 0 1 2 3
DirectionState Requests For Elevator At Floor: 0-U 1- 2-UD 3-
Controller type = Smart Controller
DoorState States For Elevator At Floor: 0-OPEN 1- 2- 3-
PressAllButtonsElevatorUser 4 - from: 0 to: 1 currently IN_ELEVATOR

Move the elevator from floor 0 to floor 1

Elevator State:
DirectionState = UP
Stopping at floor = true
Floor = 1
Number of floors = 4
Requested Floors Inside Elevator: 0 2 3
DirectionState Requests For Elevator At Floor: 0- 1-UD 2-UD 3-
Controller type = Smart Controller
DoorState States For Elevator At Floor: 0- 1-OPEN 2- 3-
PressAllButtonsElevatorUser 4 - from: 0 to: 1 currently SERVICED

Now check exception cases
Produces exception java.lang.IllegalArgumentException: Start and end floors must be different
Produces exception java.lang.IllegalArgumentException: The elevator assigned to a user must not be null
</pre>
 * @author jpaulgibson
 * @see unit_tests.elevatorUsers.JUnit_PressAllButtonsElevatorUser
 */
public class Validation_PressAllButtonsElevatorUser {
	
	 final static int NUMBER_OF_RANDOM_USERS =4;

	
	public static void main(String[] args) {
		
	ArrayList <PressAllButtonsElevatorUser>	 USERS = new ArrayList<PressAllButtonsElevatorUser>();
	
		 Random rng = SeedRNGCommandLine.getRandom(args);
			System.out.println(DateHeader.dateString());
	
		ElevatorSpecification elevator = ElevatorFactory.makeElevator();
		
		PressAllButtonsElevatorUser user = null;
		
		System.out.println("\nInitially the elevator has no user requests");
	    System.out.println(elevator);
		
		System.out.println("\nThen we add random (press all buttons) users to floors who decide which buttons to press");
        for (int i= 0;  i<NUMBER_OF_RANDOM_USERS; i++){
		  user = new PressAllButtonsElevatorUser(rng, elevator);
		  USERS.add(user);
		  user.makeDecision();
		  System.out.println(user);

        }
        
		System.out.println("\nNow the state of the elevator after these buttons have been pressed is:");
        System.out.println(elevator);
        
       System.out.println("\nAdd a new  (press all buttons) user to go from floor 0 to floor 1");
       user = new PressAllButtonsElevatorUser(0, 1, elevator);
       user.makeDecision(); // press floor buttons
       elevator.updateFloor(); // elevator decides to move up
       user.makeDecision(); // user enters elevator and presses all buttons (for floors) inside the elevator
       
       System.out.println("\nCheck that the user enters the elevator and presses all buttons inside the elevator");
       System.out.println(elevator);
       System.out.println(user);
       
       System.out.println("\nMove the elevator from floor 0 to floor 1");

       elevator.updateFloor(); // elevator moves to floor 1
       user.makeDecision(); // user exits the elevator

       System.out.println(elevator);
       System.out.println(user);
       
      
       System.out.println("\nNow check exception cases");
		
        try{ user = new PressAllButtonsElevatorUser (2,2, elevator); }
        catch (IllegalArgumentException  e) {System.out.println("Produces exception "+ e);}
        
        try{ user = new PressAllButtonsElevatorUser (1,2, null); }
        catch (IllegalArgumentException  e) {System.out.println("Produces exception "+ e);}
	}

}
