package validation_tests.abstractions;

import interfaces.ElevatorObserverSpecification;
import interfaces.ElevatorSpecification;

public abstract class Elevator_UseCase implements ElevatorObserverSpecification{
	
	final int  MAXIMUM_SCENARIO_LENGTH = get_maximum_scenario_length();
	
	int updateCounter = 0; // update count
	int floorStopCounter = 0; // floor stops
	int invariantBrokenCounter = 0; // unsafe state counter
	
	
	protected final ElevatorSpecification ELEVATOR_UNDER_TEST;
	
	/**
	 * 
	 * @return the maximum number of updates before the use case validation is terminated
	 */
	public abstract int get_maximum_scenario_length();
	
    /**
     * 
     * @param elevator is the target for the use case validation
     * @throws IllegalArgumentException if the elevator is null or if it doesn't meet the precondition for running the
     * use case as specified by {@link #check_elevator()}
     */
	public Elevator_UseCase(ElevatorSpecification elevator) throws IllegalArgumentException{
		
		if (elevator == null) throw new IllegalArgumentException("Cannot run a use case test on a null elevator object");
		
		ELEVATOR_UNDER_TEST = elevator;
		
		if (!check_elevator()) throw new IllegalArgumentException("This elevator does not meet the precondition for running the use case");
		
		elevator.connect_Observer(this);
	}
	
	/**
	 * An initial check that the elevator meets the use case precondition
	 * @return if the elevator is suitable to be checked by the use case
	 */
	public abstract boolean check_elevator ();
	
	/**
	 * This is where the use case is defined by configuring the initial state of the elevator
	 */
	protected abstract void initialise_state ();
	
	
	/**
	 * The elevator keeps updating its state until either:
	 * <ul>
	 * <li> All user requests have been met</li>
	 * <li> The maximum scenario length has been reached</li>
	 * </ul>
	 * During updates, the elevator state is output to console in human readable form.<br>
	 * At the end of the execution, the following information is displayed:
	 * <ul>
	 * <li> the number of updates </li>
	 * <li> the number of times the elevator stopped at a floor </li>
	 * <li> the number of times the elevator invariant was broken </li>
	 * </ul>
	 * The method is synchronized because it waits for a notification from {@link #elevatorUpdated()}
	 * to tell it when the case scenario is over.<br>
	 * @param elevator is the elevator on which the use case  will execute
	 * updates during the use case validation
	 */
	public synchronized void  execute(ElevatorSpecification elevator){

		initialise_state();
		elevator.elevator_logic_unit_on();
		
		// Wait until notified (by {@link #elevatorUpdated()} of the termination of the elevator use case 
		 
        try {
			wait();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		System.out.println("\n*** Execution finished ***");
		
		System.out.println("\n*** Please check that the behaviour is as required ***");
		System.out.println("The elevator was updated "+ updateCounter +" times");
		System.out.println("The elevator stopped at a floor "+ floorStopCounter +" times");
		System.out.println("The elevator was in an unsafe state  "+ invariantBrokenCounter +" times");
		
		if (invariantBrokenCounter>0) 
			System.out.println("\n*** WARNING - the elevator got into an unsafe state during the use case execution ***");

	};
	
	/**
	 * The method is synchronized because it notifies the waiting thread in
	 * {@link #execute(ElevatorSpecification elevator)}
	 * to tell it when the case scenario is over.<br>
	 */
	public synchronized void elevatorUpdated(){
		
		System.out.println("\nUpdate "+ updateCounter+ " : "+ ELEVATOR_UNDER_TEST);
		
		updateCounter++;

	    if (ELEVATOR_UNDER_TEST.stoppingAtNextFloor()) floorStopCounter++;
	    if (!ELEVATOR_UNDER_TEST.invariant()) invariantBrokenCounter++;
	    
	    if (
	    		( ELEVATOR_UNDER_TEST.noMoreRequestsAbove(0) && 
				 ELEVATOR_UNDER_TEST.noMoreRequestsBelow(ELEVATOR_UNDER_TEST.topFloor() ))
				 || 
			    (updateCounter >=  MAXIMUM_SCENARIO_LENGTH)
		   ){
	    	ELEVATOR_UNDER_TEST.elevator_logic_unit_off();
	    	
	    //  Notify the execution thread {@link # execute(ElevatorSpecification elevator)} that the use case
	    // execution is over.
	    notify();
	    }
		
		
	}

}
