package validation_tests.use_case10;

import interfaces.ElevatorSpecification;

/**
 * This is a simple use-case scenario used to test the movement of any elevator 
 * (with a minimum of {@link #MINIMUM_NUMBER_OF_FLOORS} floors).<br>
 * The scenario is executed by the single static method {@link #validate(ElevatorSpecification)}.<br>
 * It is not a comprehensive test, but is a good first scenario to walk through with the client/customer when checking
 * the required behaviour of the elevator controller logic which decides on the elevator movement.
 * The initial state of the system is as follows:
 * <ul> 
 * <li> The elevator is on the bottom floor (0) </li>
 * <li> Requests have been made inside the elevator for floors 1 and top (4) </li>
 * <li> Requests have been made on floor 2 (to go up and down), and on floor 3 (to go down) </li>
 * </ul>
 * The elevator will move until all these requests are serviced, 
 * or until a scenario limit of {@link #MAXIMUM_SCENARIO_LENGTH} is reached.<br>
 * No more requests will be made during the scenario execution.
 * After the elevator moves, we will print to the screen whether or not it stops at the next floor, and its current state.<br>
 * At the end of the execution, we print to the screen the number of floors at which the elevator stopped during the execution.
 * @see validation_tests.use_case10.Validation_Elevator_Emergency
 * @see validation_tests.use_case10.Validation_Elevator_Stupid
 * @see validation_tests.use_case10.Validation_Elevator_ImprovedStupid
 * @see validation_tests.use_case10.Validation_Elevator_Random
 * @see validation_tests.use_case10.Validation_Elevator_Smart
 *     
 * @author jpaulgibson
 */
public class Validation_Elevator_UseCase10 {
	
	public  static final int MINIMUM_NUMBER_OF_FLOORS = 5;
	public static final int MAXIMUM_SCENARIO_LENGTH = 20;
	
	/**
	 * @param elevator is the object under test by the use case scenario
	 * @throws IllegalArgumentException if the elevator has fewer than the 5 floors required by the use case
	 */
	public static void validate (ElevatorSpecification elevator) throws IllegalArgumentException{
		
		if (elevator.topFloor() < MINIMUM_NUMBER_OF_FLOORS -1)
			throw new IllegalArgumentException("This use-case scenario requires the elevator being tested to have at least 5 floors");
		
		elevator.pressFloorInElevator(1);
		elevator.pressFloorInElevator(4);

		elevator.pressUp(2);
		elevator.pressDown(2);
		elevator.pressDown(3);

		int updateCounter =0; // update count
		int floorStopCounter =0; // floor stops
		do{
			   if (elevator.updateFloor()) floorStopCounter++;
			   System.out.println(elevator);
			   updateCounter++;

			} while (!(elevator.noMoreRequestsAbove(0) & 
					   elevator.noMoreRequestsBelow(elevator.topFloor())
					  )
					 && updateCounter <  MAXIMUM_SCENARIO_LENGTH
					);
		
		System.out.println(elevator);
		if (updateCounter>= MAXIMUM_SCENARIO_LENGTH)
			     System.out.println("\n **** The requests were not completely serviced within the scenario length bound of "+ 
		                                     MAXIMUM_SCENARIO_LENGTH + " updates  *****");
	    System.out.println("\nThe elevator made "+ floorStopCounter + " floor stops until execution stopped");
	}
		
}


