package validation_tests.validation_factoriesSPL;

import enumerations.LogicType;
import enumerations.SecurityType;
import factoriesSPL.ElevatorFactory;
import interfaces.ElevatorSpecification;

import java.util.Random;

import tools.DateHeader;
import tools.SeedRNGCommandLine;

/**
 * Validation test for {@link factoriesSPL.ElevatorFactory}.<br>
 * The behaviour is random (a seed value can be provided as a command line parameter).
 * The following checks are made:
 * <ul>
 * <li> Test the default constructor</li>
 * <li> Test the random constructor (iterating {@link #NUMBER_OF_RANDOM_CONSTRUCTIONS} times)</li>
 * <li> Test the generic constructor - OK construction</li>
 * <li> Test the generic constructor - Illegal argument for number of floors being smaller than minimum</li>
 * <li> Test the generic constructor - Illegal argument for number of floors being greater than maximum</li>
 * </ul>
 *
 * Typical output:
 <pre>
he seed used for the random number generator in the test is 0.
You can override this value by passing an integer value as a main argument parameter, if you so wish.


********************************************************************
Execution Date/Time 2015/07/27 19:35:03
********************************************************************

 TEST DEFAULT CONSTRUCTOR 
 

Elevator State:
DirectionState = STAY
Stopping at floor = false
Floor = 0
Number of floors = 4
Requested Floors Inside Elevator:
DirectionState Requests For Elevator At Floor: 0- 1- 2- 3-
Controller type = Smart Controller
DoorState States For Elevator At Floor: 0-OPEN 1- 2- 3-
Invariant is true

 TEST RANDOM CONSTRUCTOR 0  
 
WARNING - The factory cannot yet build security versions

Elevator State:
DirectionState = STAY
Stopping at floor = false
Floor = 0
Number of floors = 4
Requested Floors Inside Elevator:
DirectionState Requests For Elevator At Floor: 0- 1- 2- 3-
Controller type = Smart Controller
DoorState States For Elevator At Floor: 0-OPEN 1- 2- 3-
Invariant is true

 TEST RANDOM CONSTRUCTOR 1  
 

Elevator State:
DirectionState = STAY
Stopping at floor = false
Floor = 0
Number of floors = 4
Requested Floors Inside Elevator:
DirectionState Requests For Elevator At Floor: 0- 1- 2- 3-
Controller type = Stupid Controller
Invariant is true

 TEST RANDOM CONSTRUCTOR 2  
 
WARNING - The factory cannot yet build security versions

Elevator State:
DirectionState = STAY
Stopping at floor = false
Floor = 0
Number of floors = 4
Requested Floors Inside Elevator:
DirectionState Requests For Elevator At Floor: 0- 1- 2- 3-
Controller type = Emergency Controller
Invariant is true

 TEST RANDOM CONSTRUCTOR 3  
 
WARNING - The factory cannot yet build security versions

Elevator State:
DirectionState = STAY
Stopping at floor = false
Floor = 0
Number of floors = 5
Requested Floors Inside Elevator:
DirectionState Requests For Elevator At Floor: 0- 1- 2- 3- 4-
Controller type = Random Controller
DoorState States For Elevator At Floor: 0-OPEN 1- 2- 3- 4-
Invariant is true

 TEST RANDOM CONSTRUCTOR 4  
 
WARNING - The factory cannot yet build security versions

Elevator State:
DirectionState = STAY
Stopping at floor = false
Floor = 0
Number of floors = 5
Requested Floors Inside Elevator:
DirectionState Requests For Elevator At Floor: 0- 1- 2- 3- 4-
Controller type = Smart Controller
Invariant is true

 TEST NON DEFAULT CONSTRUCTOR 
 

Elevator State:
DirectionState = STAY
Stopping at floor = false
Floor = 0
Number of floors = 2
Requested Floors Inside Elevator:
DirectionState Requests For Elevator At Floor: 0- 1-
Controller type = Stupid Controller
Invariant is true

 TEST NON DEFAULT CONSTRUCTOR EXCEPTION - NUMBER OF FLOORS TOO SMALL 
 
Correctly caught illegal argument value for number of floors = 1

 TEST NON DEFAULT CONSTRUCTOR EXCEPTION - NUMBER OF FLOORS TOO BIG 
 
Correctly caught illegal argument value for number of floors = 9

 </pre>
 * @author jpaulgibson
 * @see unit_tests.factoriesSPL.JUnit_ElevatorFactory
 */
public class Validation_ElevatorFactory {
	
	public final static int NUMBER_OF_RANDOM_CONSTRUCTIONS = 5;
	
	private static ElevatorSpecification elevator =null;
	
	public static void main(String[] args) {
		 
		final Random RNG = SeedRNGCommandLine.getRandom(args);
			System.out.println(DateHeader.dateString());
			
			System.out.println ("\n TEST DEFAULT CONSTRUCTOR \n ");
		    elevator = ElevatorFactory.makeElevator();
			System.out.println (elevator);
			System.out.println ("Invariant is "+elevator.invariant());
			
			for (int construction_count =0; construction_count < NUMBER_OF_RANDOM_CONSTRUCTIONS; construction_count++){
			  System.out.println ("\n TEST RANDOM CONSTRUCTOR "+construction_count+"  \n ");
			  elevator = ElevatorFactory.makeElevator(RNG);	
			  System.out.println (elevator);
			 System.out.println ("Invariant is "+elevator.invariant());
			}
			
			
			System.out.println ("\n TEST NON DEFAULT CONSTRUCTOR \n ");
			elevator = ElevatorFactory.makeElevator(false, 2, LogicType.STUPID, SecurityType.NONE, 1000);
			System.out.println (elevator);
			System.out.println ("Invariant is "+elevator.invariant());
			
			try{
			System.out.println ("\n TEST NON DEFAULT CONSTRUCTOR EXCEPTION - NUMBER OF FLOORS TOO SMALL \n ");
			elevator = ElevatorFactory.makeElevator(false, ElevatorFactory.MINIMUM_NUMBER_OF_FLOORS-1, LogicType.STUPID, SecurityType.NONE, 1000);
			System.out.println (elevator);
			System.out.println ("Invariant is "+elevator.invariant());}
			catch (IllegalArgumentException exception){
				System.out.println("Correctly caught illegal argument value for number of floors = "+
						(ElevatorFactory.MINIMUM_NUMBER_OF_FLOORS-1));
				         
			}
			
			try{
				System.out.println ("\n TEST NON DEFAULT CONSTRUCTOR EXCEPTION - NUMBER OF FLOORS TOO BIG \n ");
				elevator = ElevatorFactory.makeElevator(false, ElevatorFactory.MAXIMUM_NUMBER_OF_FLOORS+1, LogicType.STUPID, SecurityType.NONE, 1000);
				System.out.println (elevator);
				System.out.println ("Invariant is "+elevator.invariant());}
				catch (IllegalArgumentException exception){
					System.out.println("Correctly caught illegal argument value for number of floors = "+
							(ElevatorFactory.MAXIMUM_NUMBER_OF_FLOORS+1));
					         
				}
			
	}

}
